<?php

declare(strict_types=1);

namespace Drupal\string_plural_form\Form;

use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\string_plural_form\StringPluralFormManager;
use Drupal\string_plural_form\StringPluralFormPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure String Plural Forms settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  public function __construct(
    ConfigFactoryInterface $config_factory,
    TypedConfigManagerInterface $typedConfigManager,
    private LanguageManagerInterface $languageManager,
    private StringPluralFormManager $stringPluralFormManager,
  ) {
    parent::__construct($config_factory, $typedConfigManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('language_manager'),
      $container->get('plugin.manager.string_plural_form')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'string_plural_form_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['string_plural_form.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $languages = $this->languageManager->getLanguages(LanguageInterface::STATE_CONFIGURABLE);
    $rules = $this->stringPluralFormManager->getDefinitions();
    $options = [];
    foreach ($rules as $rule) {
      $options[$rule['id']] = $rule['admin_label'];
    }
    $saved_rules = $this->config('string_plural_form.settings')->get('rules');
    $rules_map = [];
    foreach ($languages as $language) {
      $rules_map[$language->getId()] = NULL;
      if ($saved_rules && $saved_rules[$language->getId()]) {
        $rules_map[$language->getId()] = $saved_rules[$language->getId()];
      }
    }
    $form['rules'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];
    foreach ($languages as $language) {
      $form['rules'][$language->getId()] = [
        '#type' => 'select',
        '#title' => sprintf("%s (%s)", $language->getName(), $language->getId()),
        '#default_value' => $rules_map[$language->getId()],
        '#options' => $this->getOptionsForLanguage($language->getId()),
        '#required' => TRUE,
      ];
    }
    return parent::buildForm($form, $form_state);
  }

  /**
   * Get Plural Form rules categorized by applicable to provided language.
   *
   * @param string $langcode
   *   The language code for which rules list needs to be prepared.
   *
   * @return string[]
   *   <optgroup> comptible option list of rules.
   */
  protected function getOptionsForLanguage(string $langcode): array {
    $rules = $this->stringPluralFormManager->getDefinitions();
    $recommended_rules = [];
    $other_rules = [];

    foreach ($rules as $rule) {
      $plural_form = $this->stringPluralFormManager->createInstance($rule['id']);
      if ($plural_form instanceof StringPluralFormPluginInterface) {
        if ($this->isRuleRecommendedForLanguage($plural_form, $langcode)) {
          $recommended_rules[$rule['id']] = $rule['admin_label'];
        }
        else {
          $other_rules[$rule['id']] = $rule['admin_label'];
        }
      }
    }
    $options = [
      (string) $this->t('Recommended Rules') => $recommended_rules,
      (string) $this->t('Other Rules') => $other_rules,
    ];
    return $options;
  }

  /**
   * Check if given rule is applicable for the language.
   *
   * @param \Drupal\string_plural_form\StringPluralFormPluginInterface $rule
   *   Instance of Rule.
   * @param string $langcode
   *   Language code.
   *
   * @return bool
   *   True if rule is applicable for the given language.
   */
  protected function isRuleRecommendedForLanguage(StringPluralFormPluginInterface $rule, string $langcode): bool {
    // In case "ISO 639-1" langcode is being used, then we split the incoming
    // language code to match only the language part.
    $code = explode('-', $langcode);
    $match = $code[0];
    foreach ($rule->getAssociatedLanguageCodes() as $code) {
      if ($match == $code) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $rules = $form_state->getValue('rules');
    $this->config('string_plural_form.settings')
      ->set('rules', $rules)
      ->save();
    // @codingStandardsIgnoreLine
    if (\Drupal::hasService('locale.plural.formula')) {
      foreach ($rules as $langcode => $rule_id) {
        /** @var \Drupal\string_plural_form\StringPluralFormPluginInterface $rule */
        $rule = $this->stringPluralFormManager->createInstance($rule_id);
        // @codingStandardsIgnoreLine
        \Drupal::service('locale.plural.formula')
          ->setPluralFormula($langcode, $rule->getGettextNplurals(), $rule->generateLocalePluralIndex());
      }
    }
    parent::submitForm($form, $form_state);
  }

}
